import pytest

from apiron import Endpoint, Service, ServiceBase


@pytest.fixture(scope="class")
def service():
    class SomeService(Service):
        domain = "http://foo.com"

    return SomeService


class TestServiceBase:
    def test_get_hosts_returns_empty_list_by_default(self):
        assert [] == ServiceBase.get_hosts()

    def test_required_headers_returns_empty_dict_by_default(self, service):
        assert {} == service.required_headers


class TestService:
    def test_get_hosts_returns_domain(self, service):
        assert ["http://foo.com"] == service.get_hosts()

    def test_str_method_on_class(self, service):
        assert "http://foo.com" == str(service)

    def test_repr_method_on_class(self, service):
        assert "SomeService(domain=http://foo.com)" == repr(service)

    def test_required_headers_returns_empty_dict_by_default(self, service):
        assert {} == service.required_headers

    def test_endpoints_when_no_endpoints(self, service):
        assert service.endpoints == set()

    def test_endpoints_when_one_endpoint(self, service):
        foo = Endpoint(path="/foo")
        service.foo = foo
        assert service.endpoints == {foo}

    def test_endpoints_when_multiple_endpoints(self, service):
        foo = Endpoint(path="/foo")
        bar = Endpoint(path="/bar")

        service.foo = foo
        service.bar = bar

        assert service.endpoints == {foo, bar}



import pytest

from apiron import DiscoverableService


class FakeResolver:
    @staticmethod
    def resolve(service_name):
        return ["fake"]


class FakeService(DiscoverableService):
    service_name = "fake-service"
    host_resolver_class = FakeResolver


@pytest.fixture(scope="class")
def service():
    return FakeService


class TestDiscoverableService:
    def test_get_hosts_returns_hosts_from_resolver(self, service):
        assert ["fake"] == service.get_hosts()

    def test_str_method_on_class(self, service):
        assert "fake-service" == str(service)

    def test_repr_method_on_class(self, service):
        assert "FakeService(service_name=fake-service, host_resolver=FakeResolver)" == repr(service)



from apiron.client import Timeout
from apiron.endpoint import Endpoint, JsonEndpoint, StreamingEndpoint, StubEndpoint
from apiron.exceptions import APIException, NoHostsAvailableException, UnfulfilledParameterException
from apiron.service import DiscoverableService, Service, ServiceBase

__all__ = [
    "APIException",
    "DiscoverableService",
    "Endpoint",
    "JsonEndpoint",
    "NoHostsAvailableException",
    "Service",
    "ServiceBase",
    "StreamingEndpoint",
    "StubEndpoint",
    "Timeout",
    "UnfulfilledParameterException",
]



from apiron.endpoint.endpoint import Endpoint


class StreamingEndpoint(Endpoint):
    """
    An endpoint that streams data incrementally
    """

    streaming = True

    def format_response(self, response):
        """
        Stream response in chunks

        :param requests.Response response:
            The original response from :mod:`requests`
        :return:
            The response's content
        :rtype:
            generator
        """

        return response.iter_content(chunk_size=None)



from apiron.service.base import ServiceBase


class DiscoverableService(ServiceBase):
    """
    A Service whose hosts are determined via a host resolver.
    A host resolver is any class with a :func:`resolve` method
    that takes a service name as its sole argument
    and returns a list of host names that correspond to that service.
    """

    @classmethod
    def get_hosts(cls):
        return cls.host_resolver_class.resolve(cls.service_name)

    def __str__(self):
        return self.service_name

    def __repr__(self):
        klass = self.__class__
        return "{klass}(service_name={service_name}, host_resolver={host_resolver})".format(
            klass=klass.__name__, service_name=klass.service_name, host_resolver=klass.host_resolver_class.__name__
        )



import collections
from apiron.endpoint.endpoint import Endpoint


class JsonEndpoint(Endpoint):
    """
    An endpoint that returns :mimetype:`application/json`
    """

    def __init__(
        self, *args, path="/", default_method="GET", default_params=None, required_params=None, preserve_order=False
    ):
        super().__init__(
            path=path, default_method=default_method, default_params=default_params, required_params=required_params
        )
        self.preserve_order = preserve_order

    def format_response(self, response):
        """
        Extracts JSON data from the response

        :param requests.Response response:
            The original response from :mod:`requests`
        :return:
            The response's JSON content
        :rtype:
            :class:`dict` if ``preserve_order`` is ``False``
        :rtype:
            :class:`collections.OrderedDict` if ``preserve_order`` is ``True``
        """

        return response.json(object_pairs_hook=collections.OrderedDict if self.preserve_order else None)

    @property
    def required_headers(self):
        return {"Accept": "application/json"}



import io
from unittest import mock

import pytest

from apiron import client, NoHostsAvailableException


@pytest.fixture
def mock_response():
    response = mock.Mock()
    response.history = []
    return response


@pytest.fixture
def mock_endpoint():
    endpoint = mock.Mock()
    endpoint.required_headers = {}
    endpoint.get_formatted_path.return_value = "/foo/"
    del endpoint.stub_response
    return endpoint


class TestClient:
    @mock.patch("requests.sessions.Session", autospec=True)
    def test_get_adapted_session(self, mock_session):
        adapter = mock.Mock()
        adapted_session = client.get_adapted_session(adapter)
        assert adapter == adapted_session.get_adapter("http://foo.com")
        assert adapter == adapted_session.get_adapter("https://foo.com")

    def test_get_required_headers(self, mock_endpoint):
        service = mock.Mock()
        service.required_headers = {"one": "two"}
        mock_endpoint.required_headers = {"foo": "bar"}
        expected_headers = {}
        expected_headers.update(service.required_headers)
        expected_headers.update(mock_endpoint.required_headers)
        assert expected_headers == client.get_required_headers(service, mock_endpoint)

    @mock.patch("apiron.client.requests.Request")
    @mock.patch("apiron.client.get_required_headers")
    def test_build_request_object_passes_arguments_to_request_constructor(
        self, mock_get_required_headers, mock_request_constructor, mock_endpoint
    ):
        session = mock.Mock()

        service = mock.Mock()
        service.get_hosts.return_value = ["http://host1.biz"]

        mock_endpoint.default_method = "POST"
        mock_endpoint.required_headers = {"header": "value"}
        mock_endpoint.required_params = set()

        params = {"baz": "qux"}
        mock_endpoint.get_merged_params.return_value = params
        data = "I am a data"
        files = {"file_name": io.BytesIO(b"this is a test")}
        json = {"raw": "data"}
        headers = {"Accept": "stuff"}
        cookies = {"chocolate-chip": "yes"}
        auth = mock.Mock()

        mock_get_required_headers.return_value = {"header": "value"}
        expected_headers = {}
        expected_headers.update(headers)
        expected_headers.update(mock_endpoint.required_headers)

        with mock.patch.object(session, "prepare_request") as mock_prepare_request:
            client.build_request_object(
                session,
                service,
                mock_endpoint,
                params=params,
                data=data,
                files=files,
                json=json,
                headers=headers,
                cookies=cookies,
                auth=auth,
                foo="bar",
            )

            mock_request_constructor.assert_called_once_with(
                url="http://host1.biz/foo/",
                method=mock_endpoint.default_method,
                headers=expected_headers,
                cookies=cookies,
                params=params,
                data=data,
                files=files,
                json=json,
                auth=auth,
            )

            assert 1 == mock_prepare_request.call_count

    @mock.patch("apiron.client.Timeout")
    @mock.patch("apiron.client.get_adapted_session")
    @mock.patch("apiron.client.build_request_object")
    @mock.patch("requests.adapters.HTTPAdapter", autospec=True)
    @mock.patch("requests.Session", autospec=True)
    def test_call(
        self,
        MockSession,
        MockAdapter,
        mock_build_request_object,
        mock_get_adapted_session,
        mock_timeout,
        mock_response,
        mock_endpoint,
    ):
        service = mock.Mock()
        service.get_hosts.return_value = ["http://host1.biz"]

        mock_endpoint.default_method = "GET"
        mock_endpoint.streaming = True

        request = mock.Mock()
        request.url = "http://host1.biz/foo/"
        mock_build_request_object.return_value = request

        mock_logger = mock.Mock()

        mock_response.status_code = 200
        mock_response.url = "http://host1.biz/foo/"

        mock_session = MockSession()
        mock_session.send.return_value = mock_response
        mock_get_adapted_session.return_value = mock_session

        client.call(service, mock_endpoint, timeout_spec=mock_timeout, logger=mock_logger)

        mock_get_adapted_session.assert_called_once_with(MockAdapter())
        mock_session.send.assert_called_once_with(
            request,
            timeout=(mock_timeout.connection_timeout, mock_timeout.read_timeout),
            stream=mock_endpoint.streaming,
            allow_redirects=True,
        )

        mock_logger.info.assert_any_call("GET http://host1.biz/foo/")
        mock_logger.info.assert_any_call("200 http://host1.biz/foo/")

        mock_endpoint.default_method = "POST"
        request.method = "POST"

        client.call(service, mock_endpoint, session=mock_session, timeout_spec=mock_timeout, logger=mock_logger)

        mock_session.send.assert_any_call(
            request,
            timeout=(mock_timeout.connection_timeout, mock_timeout.read_timeout),
            stream=mock_endpoint.streaming,
            allow_redirects=True,
        )

        mock_logger.info.assert_any_call("GET http://host1.biz/foo/")
        mock_logger.info.assert_any_call("200 http://host1.biz/foo/")

        request.method = "PUT"

        client.call(
            service, mock_endpoint, method="PUT", session=mock_session, timeout_spec=mock_timeout, logger=mock_logger
        )

        mock_session.send.assert_any_call(
            request,
            timeout=(mock_timeout.connection_timeout, mock_timeout.read_timeout),
            stream=mock_endpoint.streaming,
            allow_redirects=True,
        )

    @mock.patch("apiron.client.get_adapted_session")
    def test_call_with_existing_session(self, mock_get_adapted_session, mock_response, mock_endpoint):
        service = mock.Mock()
        service.get_hosts.return_value = ["http://host1.biz"]
        service.required_headers = {}

        mock_logger = mock.Mock()

        session = mock.Mock()
        session.send.return_value = mock_response

        client.call(service, mock_endpoint, session=session, logger=mock_logger)

        assert not mock_get_adapted_session.called
        assert not session.close.called

    def test_call_with_explicit_encoding(self, mock_response, mock_endpoint):
        service = mock.Mock()
        service.get_hosts.return_value = ["http://host1.biz"]
        service.required_headers = {}

        session = mock.Mock()
        session.send.return_value = mock_response

        client.call(service, mock_endpoint, session=session, logger=mock.Mock(), encoding="FAKE-CODEC")

        assert "FAKE-CODEC" == mock_response.encoding

    def test_build_request_object_raises_no_host_exception(self):
        service = mock.Mock()
        service.get_hosts.return_value = []

        with pytest.raises(NoHostsAvailableException):
            client.build_request_object(None, service, None)

    def test_choose_host_returns_one_of_the_available_hosts(self):
        hosts = ["foo", "bar", "baz"]
        service = mock.Mock()
        service.get_hosts.return_value = hosts
        assert client.choose_host(service) in hosts

    def test_choose_host_raises_exception_when_no_hosts_available(self):
        service = mock.Mock()
        service.get_hosts.return_value = []
        with pytest.raises(NoHostsAvailableException):
            client.choose_host(service)

    def test_call_when_raw_response_object_requested(self, mock_response, mock_endpoint):
        service = mock.Mock()
        service.get_hosts.return_value = ["http://host1.biz"]
        service.required_headers = {}

        session = mock.Mock()
        session.send.return_value = mock_response

        response = client.call(
            service, mock_endpoint, session=session, logger=mock.Mock(), return_raw_response_object=True
        )

        assert response is mock_response


@pytest.mark.parametrize(
    "host,path,url",
    [
        ("http://biz.com", "/endpoint", "http://biz.com/endpoint"),
        ("http://biz.com/", "endpoint", "http://biz.com/endpoint"),
        ("http://biz.com/", "/endpoint", "http://biz.com/endpoint"),
        ("http://biz.com", "endpoint", "http://biz.com/endpoint"),
        ("http://biz.com/v2", "/endpoint", "http://biz.com/v2/endpoint"),
        ("http://biz.com/v2/", "endpoint", "http://biz.com/v2/endpoint"),
        ("http://biz.com/v2/", "/endpoint", "http://biz.com/v2/endpoint"),
        ("http://biz.com/v2", "endpoint", "http://biz.com/v2/endpoint"),
    ],
)
def test_build_url(host, path, url):
    assert url == client.build_url(host, path)



import logging
import string
import warnings
from functools import partial, update_wrapper

from apiron import client
from apiron.exceptions import UnfulfilledParameterException


LOGGER = logging.getLogger(__name__)


class Endpoint:
    """
    A basic service endpoint that responds with the default ``Content-Type`` for that endpoint
    """

    def __get__(self, instance, owner):
        caller = partial(client.call, owner, self)
        update_wrapper(caller, client.call)
        return caller

    def __call__(self):
        raise TypeError("Endpoints are only callable in conjunction with a Service class.")

    def __init__(self, path="/", default_method="GET", default_params=None, required_params=None):
        """
        :param str path:
            The URL path for this endpoint, without the protocol or domain
        :param str default_method:
            (Default ``'GET'``)
            The default method to use when calling this endpoint.
        :param dict default_params:
            The default parameters to use when calling this endpoint.
            Useful when an endpoint always or most often needs a base set of parameters supplied.
        :param required_params:
            An iterable of required parameter names.
            Calling an endpoint without its required parameters raises an exception.
        """
        self.default_method = default_method

        if "?" in path:
            warnings.warn(
                f"Endpoint path ('{path}') may contain query parameters. "
                f"Use the default_params or required_params attributes in the initialization of this endpoint, "
                f"or the params argument when calling the endpoint instead.",
                stacklevel=3,
            )

        self.path = path
        self.default_params = default_params or {}
        self.required_params = required_params or set()

    def format_response(self, response):
        """
        Extracts the appropriate type of response data from a :class:`requests.Response` object

        :param requests.Response response:
            The original response from :mod:`requests`
        :return:
            The response's text content
        :rtype:
            str
        """
        return response.text

    @property
    def required_headers(self):
        """
        Generates the headers that must be sent to this endpoint based on its attributes

        :return:
            Header name, header value pairs
        :rtype:
            dict
        """
        return {}

    def get_formatted_path(self, **kwargs):
        """
        Format this endpoint's path with the supplied keyword arguments

        :return:
            The fully-formatted path
        :rtype:
            str
        """
        self._validate_path_placeholders(self.path_placeholders, kwargs)

        return self.path.format(**kwargs)

    @property
    def path_placeholders(self):
        """
        The formattable placeholders from this endpoint's path, in the order they appear.

        Example:

            >>> endpoint = Endpoint(path='/api/{foo}/{bar}')
            >>> endpoint.path_placeholders
            ['foo', 'bar']
        """

        parser = string.Formatter()
        return [placeholder_name for _, placeholder_name, _, _ in parser.parse(self.path) if placeholder_name]

    def _validate_path_placeholders(self, placeholder_names, path_kwargs):
        if any(path_kwarg not in placeholder_names for path_kwarg in path_kwargs):
            warnings.warn(
                f"An unknown path kwarg was supplied to {self}. kwargs supplied: {path_kwargs}",
                RuntimeWarning,
                stacklevel=6,
            )

    def _check_for_empty_params(self, params):
        empty_params = {param: params[param] for param in params if params[param] in (None, "")}

        if empty_params:
            warnings.warn(
                f"The {self.path} endpoint " f"was called with empty parameters: {empty_params}",
                RuntimeWarning,
                stacklevel=6,
            )

    def _check_for_unfulfilled_params(self, params):
        unfulfilled_params = {
            param for param in self.required_params if param not in params and param not in self.default_params
        }

        if unfulfilled_params:
            raise UnfulfilledParameterException(self.path, unfulfilled_params)

    def _validate_params(self, params):
        self._check_for_empty_params(params)
        self._check_for_unfulfilled_params(params)

    def get_merged_params(self, supplied_params=None):
        """
        Merge this endpoint's default parameters with the supplied parameters

        :param dict supplied_params:
            A dictionary of query parameter, value pairs
        :return:
            A dictionary of this endpoint's default parameters, merged with the supplied parameters.
            Any default parameters which have a value supplied are overridden.
        :rtype:
            dict
        :raises apiron.exceptions.UnfulfilledParameterException:
            When a required parameter for this endpoint is not a default param and is not supplied by the caller
        """
        supplied_params = supplied_params or {}

        self._validate_params(supplied_params)

        merged_params = self.default_params.copy()
        merged_params.update(supplied_params)
        return merged_params

    def __str__(self):
        return self.path

    def __repr__(self):
        return f"{self.__class__.__name__}(path='{self.path}')"



from apiron.endpoint import Endpoint


class StubEndpoint(Endpoint):
    """
    A stub endpoint designed to return a pre-baked response

    The intent is to allow for a service to be implemented
    before the endpoint is complete.
    """

    def __get__(self, instance, owner):
        return self.stub_response

    def __init__(self, stub_response=None, **kwargs):
        """
        :param stub_response:
            A pre-baked response or response-determining function.
            Pre-baked response example: ``'stub response'`` or ``{'stub': 'response'}``
            A response-determining function may operate on any arguments
            provided to the client's ``call`` method.
            Example of a response-determining function::

                def stub_response(**kwargs):
                    if kwargs.get('params') and kwargs['params'].get('param_key') == 'param_value':
                        return {'stub response': 'for param_key=param_value'}
                    else:
                        return {'default': 'response'}

        :param ``**kwargs``:
            Arbitrary parameters that can match the intended real endpoint.
            These don't do anything for the stub but streamline the interface.
        """
        super().__init__(**kwargs)
        if callable(stub_response):
            self.stub_response = stub_response
        elif stub_response:
            self.stub_response = lambda *args, **kwargs: stub_response
        else:
            self.stub_response = lambda *args, **kwargs: {"response": repr(self)}



import collections
from unittest import mock

import pytest

import apiron


@pytest.fixture
def service():
    class SomeService(apiron.Service):
        domain = "http://foo.com"

    return SomeService


@pytest.fixture
def stub_function():
    def stub_response(**kwargs):
        if kwargs.get("params") and kwargs["params"].get("param_key") == "param_value":
            return {"stub response": "stubby!"}
        else:
            return {"default": "response"}

    return stub_response


class TestEndpoint:
    def test_call(self, service):
        service.foo = apiron.Endpoint()
        service.foo()

    def test_call_without_service_raises_exception(self):
        foo = apiron.Endpoint()
        with pytest.raises(TypeError):
            foo()

    def test_default_attributes_from_constructor(self):
        foo = apiron.Endpoint()
        assert "/" == foo.path
        assert "GET" == foo.default_method

    def test_constructor_stores_passed_attributes(self):
        foo = apiron.Endpoint(path="/foo/", default_method="POST")
        assert "/foo/" == foo.path
        assert "POST" == foo.default_method

    def test_format_response(self):
        foo = apiron.Endpoint()
        mock_response = mock.Mock()
        mock_response.text = "foobar"
        assert "foobar" == foo.format_response(mock_response)

    def test_required_headers(self):
        foo = apiron.Endpoint()
        assert {} == foo.required_headers

    def test_path_placeholders_when_none_present(self):
        foo = apiron.Endpoint()
        assert [] == foo.path_placeholders

    def test_path_placeholders_when_present(self):
        foo = apiron.Endpoint(path="/foo/{one}/{two}")
        assert ["one", "two"] == foo.path_placeholders

    def test_format_path_with_correct_kwargs(self):
        foo = apiron.Endpoint(path="/{one}/{two}/")
        path_kwargs = {"one": "foo", "two": "bar"}
        assert "/foo/bar/" == foo.get_formatted_path(**path_kwargs)

    def test_format_path_with_incorrect_kwargs(self):
        foo = apiron.Endpoint(path="/{one}/{two}/")
        path_kwargs = {"foo": "bar"}
        with pytest.warns(RuntimeWarning, match="An unknown path kwarg was supplied"):
            with pytest.raises(KeyError):
                foo.get_formatted_path(**path_kwargs)

    def test_format_path_with_extra_kwargs(self):
        foo = apiron.Endpoint(path="/{one}/{two}/")
        path_kwargs = {"one": "foo", "two": "bar", "three": "not used"}
        with pytest.warns(RuntimeWarning, match="An unknown path kwarg was supplied"):
            assert "/foo/bar/" == foo.get_formatted_path(**path_kwargs)

    def test_query_parameter_in_path_generates_warning(self):
        with pytest.warns(UserWarning, match=r"Endpoint path \('/\?foo=bar'\) may contain query parameters"):
            _ = apiron.Endpoint(path="/?foo=bar")

    def test_get_merged_params(self):
        foo = apiron.Endpoint(default_params={"foo": "bar"}, required_params={"baz"})
        assert {"foo": "bar", "baz": "qux"} == foo.get_merged_params({"baz": "qux"})

    def test_get_merged_params_with_unsupplied_param(self):
        foo = apiron.Endpoint(default_params={"foo": "bar"}, required_params={"baz"})

        with pytest.raises(apiron.UnfulfilledParameterException):
            foo.get_merged_params()

    def test_get_merged_params_with_empty_param(self):
        foo = apiron.Endpoint(default_params={"foo": "bar"}, required_params={"baz"})
        with pytest.warns(RuntimeWarning, match="endpoint was called with empty parameters"):
            assert {"foo": "bar", "baz": None} == foo.get_merged_params({"baz": None})

    def test_get_merged_params_with_required_and_default_param(self):
        foo = apiron.Endpoint(default_params={"foo": "bar"}, required_params={"foo"})
        assert {"foo": "bar"} == foo.get_merged_params()

    def test_str_method(self):
        foo = apiron.Endpoint(path="/bar/baz")
        assert str(foo) == "/bar/baz"

    def test_repr_method(self):
        foo = apiron.Endpoint(path="/bar/baz")
        assert repr(foo) == "Endpoint(path='/bar/baz')"


class TestJsonEndpoint:
    def test_format_response_when_unordered(self):
        foo = apiron.JsonEndpoint()
        mock_response = mock.Mock()

        with mock.patch.object(mock_response, "json") as mock_json:
            mock_json.return_value = {"foo": "bar"}
            assert {"foo": "bar"} == foo.format_response(mock_response)
            mock_json.assert_called_once_with(object_pairs_hook=None)

    def test_format_response_when_ordered(self):
        foo = apiron.JsonEndpoint(preserve_order=True)
        mock_response = mock.Mock()

        with mock.patch.object(mock_response, "json") as mock_json:
            mock_json.return_value = {"foo": "bar"}
            assert {"foo": "bar"} == foo.format_response(mock_response)
            mock_json.assert_called_once_with(object_pairs_hook=collections.OrderedDict)

    def test_required_headers(self):
        foo = apiron.JsonEndpoint()
        assert {"Accept": "application/json"} == foo.required_headers

    def test_str_method(self):
        foo = apiron.JsonEndpoint(path="/bar/baz")
        assert str(foo) == "/bar/baz"

    def test_repr_method(self):
        foo = apiron.JsonEndpoint(path="/bar/baz")
        assert repr(foo) == "JsonEndpoint(path='/bar/baz')"


class TestStreamingEndpoint:
    def test_format_response(self):
        foo = apiron.StreamingEndpoint()
        mock_response = mock.Mock()
        assert mock_response.iter_content(chunk_size=None) == foo.format_response(mock_response)

    def test_str_method(self):
        foo = apiron.StreamingEndpoint(path="/bar/baz")
        assert str(foo) == "/bar/baz"

    def test_repr_method(self):
        foo = apiron.StreamingEndpoint(path="/bar/baz")
        assert repr(foo) == "StreamingEndpoint(path='/bar/baz')"


class TestStubEndpoint:
    def test_stub_default_response(self, service):
        service.stub_endpoint = apiron.StubEndpoint()
        assert service.stub_endpoint() == {"response": "StubEndpoint(path='/')"}

    def test_call_static(self, service):
        service.stub_endpoint = apiron.StubEndpoint(stub_response="stub response")
        assert service.stub_endpoint() == "stub response"

    @pytest.mark.parametrize(
        "test_call_kwargs,expected_response",
        [({}, {"default": "response"}), ({"params": {"param_key": "param_value"}}, {"stub response": "stubby!"})],
    )
    def test_call_dynamic(self, test_call_kwargs, expected_response, service, stub_function):
        service.stub_endpoint = apiron.StubEndpoint(stub_response=stub_function)
        actual_response = service.stub_endpoint(**test_call_kwargs)
        assert actual_response == expected_response

    def test_call_without_service_raises_exception(self):
        stub_endpoint = apiron.StubEndpoint(stub_response="foo")
        with pytest.raises(TypeError):
            stub_endpoint()

    def test_str_method(self):
        foo = apiron.StubEndpoint(path="/bar/baz")
        assert str(foo) == "/bar/baz"

    def test_repr_method(self):
        foo = apiron.StubEndpoint(path="/bar/baz")
        assert repr(foo) == "StubEndpoint(path='/bar/baz')"



from apiron.endpoint.endpoint import Endpoint
from apiron.endpoint.json import JsonEndpoint
from apiron.endpoint.streaming import StreamingEndpoint
from apiron.endpoint.stub import StubEndpoint


__all__ = ["Endpoint", "JsonEndpoint", "StreamingEndpoint", "StubEndpoint"]
